#ifndef INTERACTIVE_H_INCLUDED
#define INTERACTIVE_H_INCLUDED

#include <vector>
#include <allegro5/allegro.h>
#include <allegro5/allegro_image.h>
#include <allegro5/allegro_color.h>
#include <allegro5/allegro_font.h>
#include <allegro5/allegro_ttf.h>
#include <allegro5/allegro_primitives.h>

#include "resources.h"
#include "misc.h"
#include "key.h"
#include "joystick.h"
#include "action.h"

const double CORNER_RADIUS = 16.0;

struct IPart {
    double x;
    double y;
    double xsp;
    double ysp;
};

class Interactive;

class IPartSystem {

    public:

    IPartSystem();

    void update();

    double get_part_x(int index);
	double get_part_y(int index);
	double get_part_xsp(int index);
	double get_part_ysp(int index);
	void set_part(int index, double p_x, double p_y, double p_xsp, double p_ysp);
	int get_part_count();
	double get_part_time();
    void part_reset();

    ALLEGRO_BITMAP * get_bmp();

    static Interactive *topInteract;


    protected:

    ALLEGRO_BITMAP *bmp;

    int partn;
    int part_num, part_max;
    IPart particles[200];
    int part_count;
	int part_delay;
	int part_anim_count;
	int part_anim_delay;

};

class IButton {

	public:

	IButton(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, const char *nl, int index);

	// Return if used (key pressed while in focus)
	virtual int use(ActionManager& act, Keyboard& keys);
	virtual void update();
	// Draw the button displaying focus state
	virtual void draw(int offx, int offy, ALLEGRO_FONT *fnt, int t);

	void pass_focus(IButton *target);
	void setup_adj(IButton *up, IButton *left, IButton *right, IButton *down);
    void disable();

    int get_x();
    int get_y();
    int get_w();
    int get_h();

    void set_x(int nx);
    void set_y(int ny);
    void set_label(const char *newlabel);

	static void ol_to_coords(IButton *subject, double frac, double& x, double& y);


	protected:

	// Take focus
	void focus();
	// Loose focus
	void blur();

	int x,y;
	int w,h;
	const char *label;

    bool is_disabled;

	int id;

	Interactive *container;
	IPartSystem *partsys;

	// Pass on focus on direction key presses
	IButton *adj_up;
	IButton *adj_left;
	IButton *adj_right;
	IButton *adj_down;

	bool has_focus;

};


// --~--


class IGetVal : public IButton {

	public:

	IGetVal(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int startv, int minv, int maxv, int index);

	int use(ActionManager& act, Keyboard& keys);
	void update();
	void draw(int offx, int offy, ALLEGRO_FONT *fnt, int t);

    int get_value();
    void set_value(int newval);

    void set_submitCancel(int submit, int cancel);

	private:

	int submit_id;
	int cancel_id;

    int minval, maxval;
	int value;

};


// --~--


class IGetKey : public IButton {

	public:

	IGetKey(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int k1, int k2, const char *nl, int index);

	int use(ActionManager& act, Keyboard& keys);
	void update();
	void draw(int offx, int offy, ALLEGRO_FONT *fnt, int t);

    int get_value1();
    int get_value2();
    void set_values(int newval1, int newval2);
    KEY_TYPE test(Keyboard& keys);

    void set_submitCancel(int submit, int cancel);

	private:

	int submit_id;
	int cancel_id;

	bool in_state;
	int value1;
	int value2;
	bool conflict1;
	bool conflict2;
	const char *label;

};


// --~--


class IGetJoy : public IButton {

	public:

	IGetJoy(Interactive *con, IPartSystem *sys, int nx, int ny, int nw, int nh, int n, int a, bool p, bool s, int j, const char *nl, int index);

	int use(ActionManager& act, Keyboard& keys);
	void update();
	void draw(int offx, int offy, ALLEGRO_FONT *fnt, int t);

    void get_value(int& n, int& a, bool& p, bool& s, int& j);
    int get_sub_value();
    void set_values(int n, int a, bool p, bool s, int j);
    int test(Joystick *joy);

    void set_submitCancel(int submit, int cancel);

	private:

	int submit_id;
	int cancel_id;

    int jid;
    bool isStick;
	bool in_state;
	int num, axe;
	bool plus;
	bool conflict;
	//const char *label;

};


// --~--


class Interactive {

	public:

	Interactive(IPartSystem *sys, int nx, int ny, int nw, int nh, const char *nt);
	~Interactive();

	// Fix buttons
	int update(ActionManager& act, Keyboard& inp_keyboard);
	IButton * add_button(IPartSystem * sys, int nx, int ny, int nw, int nh, const char *nl, int index);
	IGetVal * add_getval(IPartSystem * sys, int nx, int ny, int nw, int nh, int startv,int minv,int maxv, int index);
	IGetKey * add_getkey(IPartSystem * sys, int nx, int ny, int nw, int nh, int k1, int k2, const char *nl, int index);
	IGetJoy * add_getjoy(IPartSystem * sys, int nx, int ny, int nw, int nh, int n, int a, bool p, bool s, int j, const char *nl, int index);
	void set_initial_button(IButton *but);
	void set_adjacent_buttons(IButton *button, IButton *up, IButton *left, IButton *right, IButton *down);
	// Show myself
	//void draw(bool border, bool background, bool title);
	void draw(int timing);

	void set_used_but(IButton *but);
	IButton * get_used_but();

    void set_x(int nx);
    void set_y(int ny);
    void set_w(int nw);
    void set_h(int nh);

    int get_x();
    int get_y();
    int get_w();
    int get_h();
    const char * get_title();


	private:

    Interactive *prevInteract;
    IPartSystem *partsys;

	int x,y;
	int w,h;
	const char *title;
	ALLEGRO_FONT *fnt;


	int cycle_count;
	int cycle_delay;

	IButton * button_in_use;

	std::vector<IButton*> list_buttons;

};

#endif // INTERACTIVE_H_INCLUDED
